-- A simulator for the external file search order

Parse Source . . myself
myDir = myself~Left( myself~lastPos(.File~separator) -1)

Parse Upper Arg options

-- Handle requests for help
uopt = options~strip
If uopt == "?" | uopt == "-?" | uopt == "/?" | -
   uopt~abbrev("-help",2) | uopt~abbrev("/help",2) Then Do
  Say .resources~help
  Exit 1
End   

-- Process options and store the results in .local
-- If no interpreter was specified in the command line, use ooRexx
If options = "" Then targetInterpreter = "OOREXX"
Else Do option Over options~space~makeArray(" ")
  Select
    When option~abbrev("OBJREXX", 3) Then targetInterpreter = "OBJREXX"
  	When option~abbrev("REXXSAA", 5) Then targetInterpreter = "REXXSAA"
	When option~abbrev("REGINA",  3) Then targetInterpreter = "REGINA"
	When option~abbrev("OOREXX",  3) Then targetInterpreter = "OOREXX"
    Otherwise                      
      Say "Unrecognized option '"option"'."
	  Exit 1
  End	  
End

.local~targetInterpreter = targetInterpreter

.local~mySeparator       = .File~separator

-- Every interpreter has its own set of exceptions
Select
  When .targetInterpreter = "OOREXX" Then Do
    .local~ExtensionException    = .Routines~ooRexx.HasExtension
	.local~DirectoryException    = .Routines~ooRexx.DirectoryException
	.local~extensionFirst        = 1
	.local~DefaultDirectoryIndex = 2
	End
  When .targetInterpreter = "REGINA" Then Do
    .local~ExtensionException    = .Routines~Regina.HasExtension
	.local~DirectoryException    = .Routines~Regina.DirectoryException
	.local~extensionFirst        = 0
	.local~DefaultDirectoryIndex = 1
	End
  Otherwise
    Say "Unsupported interpreter combination, '".targetInterpreter"'."
	Exit 1
End

s = .mySeparator -- Will need a lot of these below

If .File~new(myDir||s"sotest.rex")~exists Then Do
  Call Directory mydir||s"subdir"s"dotdotcurr"s"curr"
  Select
    When .targetInterpreter == "OOREXX" Then Do
      .local~superPath  = .Array~of( mydir||s"subdir"s"dotdotsame"s"same", ".", mydir||s"subdir"s"dotdotpath"s"path" )		  
      .local~extensions = .Array~of(".rex")
      .extensions~append("")
	  End
	When .targetInterpreter == "REGINA" Then Do
	  -- Assume REGINA_MACROS is empty
	  .local~superPath = .Array~of( ".", mydir||s"subdir"s"dotdotpath"s"path" )		
      -- Assume REGINA_SUFFIXES is empty
	  .local~extensions = ("", ".rexx", ".rex", ".cmd", ".rx")
	  End
  End
  End
Else Do
  Say "Unsupported configuration."
  Exit 1
End
 
Do counter i fn Over .Routines~TestCases[]
  found = ExternalSearchOrder(fn)
  Say "Pass."Left(i,2) "=" (found == "")~?(".false",".true ")"; Pass."i".test = '"fn"'"
End

Say "Pass.0 =" i
Say "Return Pass."

Exit

-------------------------------------------------------------------------------


-------------------------------------------------------------------------------

::Routine ExternalSearchOrder
  Use Arg fn

  If .ExtensionException[fn] Then extensionsToCheck  = .Array~of( "" )
  Else                            extensionsToCheck  = .extensions~copy
  
  If .DirectoryException[fn] Then directoriesToCheck = .superPath[.defaultDirectoryIndex]
  Else                            directoriesToCheck = .superPath~copy

  If .extensionFirst Then
    Do   ext over extensionsToCheck  Until fSpec \== ""
      Do dir over directoriesToCheck Until fSpec \== ""
	    fSpec = CheckFile(dir, fn, ext)
	  End
    End
  Else
    Do   dir over directoriesToCheck Until fSpec \== ""
      Do ext over extensionsToCheck  Until fSpec \== ""
	    fSpec = CheckFile(dir, fn, ext)
	  End
    End
  
  Return fSpec

-------------------------------------------------------------------------------
-- CheckFile -- Check whether a specified file exists, according to our      --
-- composition algorithm                                                     --
-------------------------------------------------------------------------------

::Routine CheckFile
  Parse arg dir, fn, ext
  file = .File~new( Compose(dir,fn,ext) )
  If file~exists, file~isFile Then Return file~absolutePath
  Return ""

-------------------------------------------------------------------------------
-- Compose -- File composition algorithm. Given a (possibly partial or       --
-- empty) path, a (relative or absolute) file name and an optional           --
-- extension, return a file name. The composition algorthms ignores          --
-- the 'directory' argument when the filename incurs in a system-            --
-- and interpreter-dependent exception mechanism.                            --
-------------------------------------------------------------------------------

::Routine Compose
  Parse arg directory, fn, ext
  If .DirectoryException[fn] Then Return fn || ext
  If \directory~endsWith(.mySeparator) Then directory ||= .mySeparator
  Return directory || fn || ext
  
-------------------------------------------------------------------------------
-- ooRexx-specific routines                                                  --
-------------------------------------------------------------------------------

::Routine ooRexx.HasExtension
  Parse Arg fileName
  Do c over filename~reverse~makeArray("")
    If c == .mySeparator Then Return .false
	If c == "."          Then Return .true
  End
  Return .false
  
::Routine ooRexx.DirectoryException  
  Parse Arg fileName
  Return -  -- SysFileSystem::hasDirectory in interpreter/platform/unix/SysFileSystem.cpp
    filename[1] == '~'     | -  -- User directories
    filename[1] == '/'     | -  -- Root directory
	filename[1,2] == "./"  | -  -- This directory
	filename[1,3] == "../" | -  -- Parent directory
    | -      -- SysFileSystem::hasDirectory in interpreter\platform\windows\SysFileSystem.cpp 
    fileName[1] == "\"     | -  -- Relative to the current drive    
    filename[2] == ":"     | -  -- May be relative, i.e. D:dir/file
	filename[1,2] == ".\"  | -  -- This directory
	filename[1,3] == "..\"      -- Parent directory

-------------------------------------------------------------------------------
-- Regina-specific routines                                                  --
-------------------------------------------------------------------------------

::Routine Regina.HasExtension
  Parse Arg fileName
  Do c over filename~reverse~makeArray("")
    If c == .mySeparator Then Return .false
	If c == "."          Then Return .true
  End
  Return .false
  
::Routine Regina.DirectoryException  
  Parse Arg fileName
  Return filename~contains(mySeparator)

-------------------------------------------------------------------------------
-- TestCases -- Returns the list of test cases as an array                   --
-------------------------------------------------------------------------------

::Routine TestCases Public
  s = .mySeparator
  Return -
  (-
    "same same.rex curr curr.rex path path.rex" -
    "lib"s"samelib lib"s"samelib.rex lib"s"currlib lib"s"currlib.rex lib"s"pathlib lib"s"pathlib.rex" -
	"."s"same ."s"same.rex ."s"curr ."s"curr.rex ."s"path ."s"path.rex" -
	".."s"dotdotsame .."s"dotdotsame.rex .."s"dotdotcurr .."s"dotdotcurr.rex .."s"dotdotpath .."s"dotdotpath.rex" -
	"lib"s".."s".."s"dotdotsame lib"s".."s".."s"dotdotsame.rex lib"s".."s".."s"dotdotcurr lib"s".."s".."s"dotdotcurr.rex lib"s".."s".."s"dotdotpath lib"s".."s".."s"dotdotpath.rex" -
  )~makeArray(" ")

-------------------------------------------------------------------------------
-- HELP ::Resource                                                           --
-------------------------------------------------------------------------------
	
::Resource Help
SIMUL.REX -- A simulator for the external file search order

Syntax:

  simul [filename] [ ( options [)] ]
  
Where:

  filename is TBD
           If filename contains blanks, it has to be enclosed 
		   between double quotes.
		   
Options:  		   

  OBJrexx  Simulate results for the Object Rexx interpreter
  OORexx   Simulate results for the ooRexx interpreter
  REGina   Simulate results for the Regina interpreter
  REXXSaa  Simulate results for the REXXSAA interpreter

The operating system defaults to the operating system being used, and the
same is true for the interpreter. If conflicting options are specified,
the last one is used.
::END